#include <cassert>
#include <generator>
#include <iostream>

// emulate Python's range() using coroutines
// needs gcc ≥ 14 with -std=c++23

std::generator<int> p_range(int start, int stop, int step = 1) {
	assert(step != 0);
	int i = start;
	auto cmp = step > 0
		? [](int a, int b) { return a < b; }
		: [](int a, int b) { return a > b; };

	while (cmp(i, stop)) {
		co_yield i;
		i += step;
	}
}

std::generator<int> p_range(int stop) {
	return p_range(0, stop);
}

int main() {
	for (int i : p_range(7)) {
		std::cout << i << ' ';
	}
	std::cout << '\n';

	for (int i : p_range(3, 100, 7)) {
		std::cout << i << ' ';
	}
	std::cout << '\n';
}
